@@ -2,7 +2,7 @@ from keras import backend as K
 from keras.legacy import interfaces
 from keras.optimizers import Optimizer
 from .utils import _init_weight_decays, _apply_weight_decays, _check_args
-from .utils import _apply_lr_multiplier, _update_t_cur_eta_t
+from .utils import _apply_lr_multiplier, _update_t_cur_eta_t, _set_autorestart
 
 
 class AdamW(Optimizer):
@@ -19,7 +19,6 @@ class AdamW(Optimizer):
             extracts weight penalties from layers, and overrides `weight_decays`.
         zero_penalties: bool. If True and `model` is passed, will zero weight
             penalties (loss-based). (RECOMMENDED; see README "Use guidelines").
-        batch_size:       int >= 1. Train input batch size; used for normalization
         total_iterations: int >= 0. Total expected iterations / weight updates
                           throughout training, used for normalization; <1>
         lr_multipliers:   dict / None. Name-value pairs specifying per-layer lr
@@ -30,6 +29,13 @@ class AdamW(Optimizer):
         use_cosine_annealing: bool. If True, multiplies lr each train iteration
                               as a function of eta_min, eta_max, total_iterations,
                               and t_cur (current); [2]-Appendix, 2
+        autorestart: bool / None. If True, will automatically do Warm Restarts
+                     by resetting `t_cur=0` after `total_iterations`. If None,
+                     will default to same as `use_cosine_annealing`. If True
+                     but `use_cosine_annealing` is False, will raise ValueError.
+                     Note: once optimizer is built (happens on first model fit),
+                     changing `autorestart` has no effect; optimizer needs to be
+                     re-built.
         eta_min, eta_max: int, int. Min & max values of cosine annealing
                           lr multiplier; [2]-Appendix, 2
         t_cur: int. Value to initialize t_cur to - used for 'warm restarts'.
@@ -56,9 +62,9 @@ class AdamW(Optimizer):
     """
     def __init__(self, lr=0.001, beta_1=0.9, beta_2=0.999, amsgrad=False,
                  epsilon=None, decay=0.0, model=None, zero_penalties=True,
-                 batch_size=32, total_iterations=0, total_iterations_wd=None,
+                 total_iterations=0, total_iterations_wd=None,
                  use_cosine_annealing=False, lr_multipliers=None,
-                 weight_decays=None, init_verbose=True,
+                 weight_decays=None, autorestart=None, init_verbose=True,
                  eta_min=0, eta_max=1, t_cur=0, **kwargs):
         if total_iterations > 1:
             weight_decays = _init_weight_decays(model, zero_penalties,
@@ -79,7 +85,6 @@ class AdamW(Optimizer):
 
         self.initial_decay = decay
         self.epsilon = epsilon or K.epsilon()
-        self.batch_size = batch_size
         self.total_iterations = total_iterations
         self.total_iterations_wd = total_iterations_wd or total_iterations
         self.amsgrad = amsgrad
@@ -88,6 +93,7 @@ class AdamW(Optimizer):
         self.init_verbose = init_verbose
         self.use_cosine_annealing = use_cosine_annealing
 
+        _set_autorestart(self, autorestart, use_cosine_annealing)
         _check_args(self, total_iterations, use_cosine_annealing, weight_decays)
         self._init_lr = lr  # to print lr_mult setup
         self._init_notified = False
@@ -123,16 +129,17 @@ class AdamW(Optimizer):
 
             m_t = (self.beta_1 * m) + (1. - self.beta_1) * g
             v_t = (self.beta_2 * v) + (1. - self.beta_2) * K.square(g)
-            if self.amsgrad:
-                vhat_t = K.maximum(vhat, v_t)
-                p_t = p - lr_t * m_t / (K.sqrt(vhat_t) + self.epsilon)
-                self.updates.append(K.update(vhat, vhat_t))
-            else:
-                p_t = p - lr_t * m_t / (K.sqrt(v_t) + self.epsilon)
-
             self.updates.append(K.update(m, m_t))
             self.updates.append(K.update(v, v_t))
 
+            if self.amsgrad:
+                vhat_t = K.maximum(vhat, v_t)
+                p_t = p - self.eta_t * lr_t * m_t / (
+                    K.sqrt(vhat_t) + self.epsilon)
+                self.updates.append(K.update(vhat, vhat_t))
+            else:
+                p_t = p - self.eta_t * lr_t * m_t / (K.sqrt(v_t) + self.epsilon)
+
             # Weight decays
             if p.name in self.weight_decays.keys():
                 p_t = _apply_weight_decays(self, p, p_t)
@@ -156,11 +163,11 @@ class AdamW(Optimizer):
             'beta_1': float(K.get_value(self.beta_1)),
             'beta_2': float(K.get_value(self.beta_2)),
             'decay': float(K.get_value(self.decay)),
-            'batch_size': int(self.batch_size),
             'total_iterations': int(self.total_iterations),
             'weight_decays': self.weight_decays,
             'lr_multipliers': self.lr_multipliers,
             'use_cosine_annealing': self.use_cosine_annealing,
+            'autorestart': self.autorestart,
             't_cur': int(K.get_value(self.t_cur)),
             'eta_t': float(K.eval(self.eta_t)),
             'eta_min': float(K.get_value(self.eta_min)),
@@ -192,7 +199,6 @@ class NadamW(Optimizer):
             extracts weight penalties from layers, and overrides `weight_decays`.
         zero_penalties: bool. If True and `model` is passed, will zero weight
             penalties (loss-based). (RECOMMENDED; see README "Use guidelines").
-        batch_size:       int >= 1. Train input batch size; used for normalization
         total_iterations: int >= 0. Total expected iterations / weight updates
                           throughout training, used for normalization; <1>
         lr_multipliers:   dict / None. Name-value pairs specifying per-layer lr
@@ -203,6 +209,13 @@ class NadamW(Optimizer):
         use_cosine_annealing: bool. If True, multiplies lr each train iteration
                               as a function of eta_min, eta_max, total_iterations,
                               and t_cur (current); [3]-Appendix, 2
+        autorestart: bool / None. If True, will automatically do Warm Restarts
+                     by resetting `t_cur=0` after `total_iterations`. If None,
+                     will default to same as `use_cosine_annealing`. If True
+                     but `use_cosine_annealing` is False, will raise ValueError.
+                     Note: once optimizer is built (happens on first model fit),
+                     changing `autorestart` has no effect; optimizer needs to be
+                     re-built.
         eta_min, eta_max: int, int. Min & max values of cosine annealing
                           lr multiplier; [3]-Appendix, 2
         t_cur: int. Value to initialize t_cur to - used for 'warm restarts'.
@@ -231,10 +244,10 @@ class NadamW(Optimizer):
     """
     def __init__(self, lr=0.002, beta_1=0.9, beta_2=0.999,
                  schedule_decay=0.004, epsilon=None,
-                 model=None, zero_penalties=True, batch_size=32,
+                 model=None, zero_penalties=True,
                  total_iterations=0, total_iterations_wd=None,
                  use_cosine_annealing=False, lr_multipliers=None,
-                 weight_decays=None, init_verbose=True,
+                 weight_decays=None, autorestart=None, init_verbose=True,
                  eta_min=0, eta_max=1, t_cur=0, **kwargs):
         if total_iterations > 1:
             weight_decays = _init_weight_decays(model, zero_penalties,
@@ -255,7 +268,6 @@ class NadamW(Optimizer):
 
         self.epsilon = epsilon or K.epsilon()
         self.schedule_decay = schedule_decay
-        self.batch_size = batch_size
         self.total_iterations = total_iterations
         self.total_iterations_wd = total_iterations_wd or total_iterations
         self.lr_multipliers = lr_multipliers
@@ -263,6 +275,7 @@ class NadamW(Optimizer):
         self.use_cosine_annealing = use_cosine_annealing
         self.init_verbose = init_verbose
 
+        _set_autorestart(self, autorestart, use_cosine_annealing)
         _check_args(self, total_iterations, use_cosine_annealing, weight_decays)
         self._init_lr = lr  # to print lr_mult setup
         self._init_notified = False
@@ -306,8 +319,9 @@ class NadamW(Optimizer):
 
             self.updates.append(K.update(m, m_t))
             self.updates.append(K.update(v, v_t))
+
             p_t = p - self.eta_t * lr_t * m_t_bar / (
-                    K.sqrt(v_t_prime) + self.epsilon)
+                K.sqrt(v_t_prime) + self.epsilon)
 
             # Weight decays
             if p.name in self.weight_decays.keys():
@@ -333,11 +347,11 @@ class NadamW(Optimizer):
             'beta_2': float(K.get_value(self.beta_2)),
             'epsilon': self.epsilon,
             'schedule_decay': self.schedule_decay,
-            'batch_size': int(self.batch_size),
             'total_iterations': int(self.total_iterations),
             'weight_decays': self.weight_decays,
             'lr_multipliers': self.lr_multipliers,
             'use_cosine_annealing': self.use_cosine_annealing,
+            'autorestart': self.autorestart,
             't_cur': int(K.get_value(self.t_cur)),
             'eta_t': float(K.eval(self.eta_t)),
             'eta_min': float(K.get_value(self.eta_min)),
@@ -365,7 +379,6 @@ class SGDW(Optimizer):
             extracts weight penalties from layers, and overrides `weight_decays`.
         zero_penalties: bool. If True and `model` is passed, will zero weight
             penalties (loss-based). (RECOMMENDED; see README "Use guidelines").
-        batch_size:       int >= 1. Train input batch size; used for normalization
         total_iterations: int >= 0. Total expected iterations / weight updates
                           throughout training, used for normalization; <1>
         lr_multipliers:   dict / None. Name-value pairs specifying per-layer lr
@@ -376,6 +389,13 @@ class SGDW(Optimizer):
         use_cosine_annealing: bool. If True, multiplies lr each train iteration
                               as a function of eta_min, eta_max, total_iterations,
                               and t_cur (current); [2]-Appendix, 2
+        autorestart: bool / None. If True, will automatically do Warm Restarts
+                     by resetting `t_cur=0` after `total_iterations`. If None,
+                     will default to same as `use_cosine_annealing`. If True
+                     but `use_cosine_annealing` is False, will raise ValueError.
+                     Note: once optimizer is built (happens on first model fit),
+                     changing `autorestart` has no effect; optimizer needs to be
+                     re-built.
         eta_min, eta_max: int, int. Min & max values of cosine annealing
                           lr multiplier; [2]-Appendix, 2
         t_cur: int. Value to initialize t_cur to - used for 'warm restarts'.
@@ -402,10 +422,10 @@ class SGDW(Optimizer):
          (https://arxiv.org/abs/1711.05101)
     """
     def __init__(self, lr=0.01, momentum=0., nesterov=False, decay=0.0,
-                 model=None, zero_penalties=True, batch_size=32,
+                 model=None, zero_penalties=True,
                  total_iterations=0, total_iterations_wd=None,
                  use_cosine_annealing=False, lr_multipliers=None,
-                 weight_decays=None, init_verbose=True,
+                 weight_decays=None, autorestart=None, init_verbose=True,
                  eta_min=0, eta_max=1, t_cur=0, **kwargs):
         if total_iterations > 1:
             weight_decays = _init_weight_decays(model, zero_penalties,
@@ -424,7 +444,6 @@ class SGDW(Optimizer):
             self.t_cur = K.variable(t_cur, dtype='int64', name='t_cur')
 
         self.initial_decay = decay
-        self.batch_size = batch_size
         self.total_iterations = total_iterations
         self.total_iterations_wd = total_iterations_wd or total_iterations
         self.nesterov = nesterov
@@ -433,6 +452,7 @@ class SGDW(Optimizer):
         self.init_verbose = init_verbose
         self.use_cosine_annealing = use_cosine_annealing
 
+        _set_autorestart(self, autorestart, use_cosine_annealing)
         _check_args(self, total_iterations, use_cosine_annealing, weight_decays)
         self._init_lr = lr  # to print lr_mult setup
         self._init_notified = False
@@ -488,11 +508,11 @@ class SGDW(Optimizer):
             'momentum': float(K.get_value(self.momentum)),
             'decay': float(K.get_value(self.decay)),
             'nesterov': self.nesterov,
-            'batch_size': int(self.batch_size),
             'total_iterations': int(self.total_iterations),
             'weight_decays': self.weight_decays,
             'lr_multipliers': self.lr_multipliers,
             'use_cosine_annealing': self.use_cosine_annealing,
+            'autorestart': self.autorestart,
             't_cur': int(K.get_value(self.t_cur)),
             'eta_t': float(K.eval(self.eta_t)),
             'eta_min': float(K.get_value(self.eta_min)),
