@@ -3,8 +3,8 @@ import numpy as np
 import tensorflow as tf
 from termcolor import colored
 from tensorflow.python import ops
-from tensorflow.python.ops import math_ops, state_ops
-
+from tensorflow.python.ops import math_ops, state_ops, control_flow_ops
+from . import TF_KERAS
 
 WARN = colored('WARNING:', 'red')
 
@@ -16,7 +16,7 @@ def _apply_weight_decays(self, var, var_t):
             print("Both penalties are 0 for %s, will skip" % var.name)
         return var_t
 
-    norm = math_ops.cast(math_ops.sqrt(self.batch_size / self.total_iterations_wd),
+    norm = math_ops.cast(math_ops.sqrt(1 / self.total_iterations_wd),
                          'float32')
     l1_normalized = l1 * norm
     l2_normalized = l2 * norm
@@ -30,7 +30,7 @@ def _apply_weight_decays(self, var, var_t):
     var_t = var_t - self.eta_t * decay
 
     if self.init_verbose and not self._init_notified:
-        norm_print = (self.batch_size / self.total_iterations_wd) ** (1 / 2)
+        norm_print = (1 / self.total_iterations_wd) ** (1 / 2)
         l1n_print, l2n_print = l1 * norm_print, l2 * norm_print
         decays_str = "{}(L1), {}(L2)".format(l1n_print, l2n_print)
         print('{} weight decay set for {}'.format(decays_str, var.name))
@@ -60,13 +60,13 @@ def _apply_lr_multiplier(self, lr_t, var):
             print('{} init learning rate set for {} -- {}'.format(
                '%.e' % round(lr_print, 5), var.name, lr_t))
         else:
-            print('No change in learning rate {} -- {}'.format(var.name,
-                                                               lr_print))
+            print('No change in learning rate {} -- {}'.format(
+                var.name, lr_print))
     return lr_t
 
 
 def _update_t_cur_eta_t(self):  # keras
-    self.updates.append(state_ops.assign_add(self.t_cur, 1))
+    self.updates.append(_update_t_cur(self))
     # Cosine annealing
     if self.use_cosine_annealing:
         # ensure eta_t is updated AFTER t_cur
@@ -75,14 +75,13 @@ def _update_t_cur_eta_t(self):  # keras
                                                  _compute_eta_t(self)))
 
 
-def _update_t_cur_eta_t_apply_lr_mult(self, lr_t=None, var=None):  # tf.keras
+def _update_t_cur_eta_t_v2(self, lr_t=None, var=None):  # tf.keras
     t_cur_update, eta_t_update = None, None  # in case not assigned
 
     # update `t_cur` if iterating last `(grad, var)`
-    iteration_done = self._updates_processed == (self._updates_per_iter - 1)
+    iteration_done = (self._updates_processed == (self._updates_per_iter - 1))
     if iteration_done:
-        t_cur_update = state_ops.assign_add(self.t_cur, 1,
-                                            use_locking=self._use_locking)
+        t_cur_update = _update_t_cur(self)
         self._updates_processed = 0  # reset
     else:
         self._updates_processed += 1
@@ -93,13 +92,32 @@ def _update_t_cur_eta_t_apply_lr_mult(self, lr_t=None, var=None):  # tf.keras
         with ops.control_dependencies([t_cur_update]):
             eta_t_update = state_ops.assign(self.eta_t, _compute_eta_t(self),
                                             use_locking=self._use_locking)
-    # Learning rate multipliers
-    if self.lr_multipliers is not None:
-        lr_t = _apply_lr_multiplier(self, lr_t, var)
+        self.lr_t = lr_t * self.eta_t  # for external tracking
 
     return iteration_done, t_cur_update, eta_t_update
 
 
+def _update_t_cur(self):
+    kw = {'use_locking': self._use_locking} if TF_KERAS else {}
+    if self.autorestart:
+        return control_flow_ops.cond(
+            math_ops.equal(self.t_cur, self.total_iterations - 1),
+            lambda: state_ops.assign(self.t_cur, 0, **kw),
+            lambda: state_ops.assign_add(self.t_cur, 1, **kw),
+        )
+    return state_ops.assign_add(self.t_cur, 1, **kw)
+
+
+def _set_autorestart(self, autorestart, use_cosine_annealing):
+    if autorestart is None:
+        self.autorestart = bool(use_cosine_annealing)
+    elif autorestart and not use_cosine_annealing:
+        raise ValueError("`autorestart` can only be used with "
+                         "`use_cosine_annealing`")
+    else:
+        self.autorestart = autorestart
+
+
 def _check_args(self, total_iterations, use_cosine_annealing, weight_decays):
     if use_cosine_annealing and total_iterations > 1:
         print('Using cosine annealing learning rates')
@@ -108,6 +126,7 @@ def _check_args(self, total_iterations, use_cosine_annealing, weight_decays):
               + " to use cosine annealing and/or weight decays; "
               "proceeding without either")
         self.use_cosine_annealing = False
+        self.autorestart = False
         self.weight_decays = {}
 
 
